#ifndef LINKEDLIST_H
#define LINKEDLIST_H



template <class dataType>
class LinkedList
{

// ========================================
// List Internal Classes:
// ========================================
protected:

    // Simple node container class.
    template <class dataType>
    class listNode
    {
    public:
        dataType data;                  // the data stored in the node
        listNode<dataType>* previous;   // the previous node in the list
        listNode<dataType>* next;       // the next node in the list
    };

    // define a shortcut for accessing a node of the same type as the
    // list type, since we are obviously going to use nodes of the
    // same paramaterized type in the list.
    typedef listNode<dataType> node;


// ========================================
// List Public Interface:
// ========================================
public:

// ----------------------------------------------------------------
//  Name:           LinkedList::LinkedList
//  Description:    Constructor, creates empty linked list.
//  Arguments:      None
//  Return Value:   None.
// ----------------------------------------------------------------
    LinkedList()
    {
        head = 0;
        tail = 0;
        current = 0;
    }


// ----------------------------------------------------------------
//  Name:           LinkedList::~List
//  Description:    Destructor, deletes every node in list.
//  Arguments:      None
//  Return Value:   None.
// ----------------------------------------------------------------
    ~LinkedList()
    {
        clear();
    }


// ----------------------------------------------------------------
//  Name:           LinkedList::isEmpty
//  Description:    determines if the list is empty or not.
//  Arguments:      None
//  Return Value:   True if list is empty, False otherwise.
// ----------------------------------------------------------------
    bool isEmpty()
    {
        if( head == 0 )
            return true;
        return false;
    }


// ----------------------------------------------------------------
//  Name:           LinkedList::isValid
//  Description:    Determines if the current iterator is valid 
//                  (points to an existing node).
//  Arguments:      None
//  Return Value:   True if the iterator is valid, False otherwise.
// ----------------------------------------------------------------
    bool isValid()
    {
        if( current != 0 )
            return true;
        return false;
    }


// ----------------------------------------------------------------
//  Name:           LinkedList::start
//  Description:    Moves the iterator to the start of the list.
//  Arguments:      None.
//  Return Value:   None.
// ----------------------------------------------------------------
    void start()
    {
        current = head;
    }

    
// ----------------------------------------------------------------
//  Name:           LinkedList::end
//  Description:    Moves the iterator to the end of the list.
//  Arguments:      None.
//  Return Value:   None.
// ----------------------------------------------------------------
    void end()
    {
        current = tail;   
    }


// ----------------------------------------------------------------
//  Name:           LinkedList::forth
//  Description:    Moves the iterator to the next node in the list, or
//                  invalidates iterator if it is already at end.
//  Arguments:      None.
//  Return Value:   None.
// ----------------------------------------------------------------
    void forth()
    {
        if( current != 0 )
            current = current->next;
    }

    
// ----------------------------------------------------------------
//  Name:           LinkedList::back
//  Description:    Moves the iterator to the previous node in the list, or
//                  invalidates iterator if it is already at start.
//  Arguments:      None.
//  Return Value:   None.
// ----------------------------------------------------------------
    void back()
    {
        if( current != 0 )
            current = current->previous;
    }


// ----------------------------------------------------------------
//  Name:           LinkedList::append
//  Description:    Appends a data item to the end of the list.
//  Arguments:      data: data to insert into list.
//  Return Value:   None.
// ----------------------------------------------------------------
    void append( dataType& data )
    {
        node* temp = new node;
        temp->data = data;
        temp->next = 0;
        temp->previous = tail;
        if( tail != 0 )
            tail->next = temp;

        tail = temp;
    
        if( head == 0 )
            head = temp;
    }

    

// ----------------------------------------------------------------
//  Name:           LinkedList::remove
//  Description:    remove's the data item at the current node, or does
//                  nothing if current is invalid. Current is always invalid
//                  after this routine is called.
//  Arguments:      None.
//  Return Value:   None.
// ----------------------------------------------------------------
    void remove()
    {
        if( current != 0 )
        {
            if( current == head )
                head = current->next;
            if( current == tail )
                tail = current->previous;
            if( current->previous != 0 )
                current->previous->next = current->next;
            if( current->next != 0 )
                current->next->previous = current->previous;
            delete current;
            current = 0;
        }
    }


// ----------------------------------------------------------------
//  Name:           LinkedList::clear
//  Description:    Deletes every node in the list.
//  Arguments:      None.
//  Return Value:   None.
// ----------------------------------------------------------------
    void clear()
    {
        node* temp = head;
        while( temp != 0 )
        {
            current = temp->next;
            delete temp;
            temp = current;
        }
        current = 0;
        head = 0;
        tail = 0;
    }


// ----------------------------------------------------------------
//  Name:           LinkedList::item
//  Description:    Gets the item at the current iterator index. It is
//                  reccomended that a isValid() check be performed before
//                  this is executed.
//  Arguments:      None.
//  Return Value:   the data item at the current iterator's node.
// ----------------------------------------------------------------
    dataType& item()
    {
        return current->data;
    }


// ========================================
// List Data Members:
// ========================================
protected:

    node* head;                 // pointer to the first node in the list.
    node* tail;                 // pointer to the last node in the list.
    node* current;              // internal iterator which retains the
                                // current node for easy access.
};

#endif